@@ -426,6 +426,9 @@ $ %(prog)s init ~/fuerte /opt/ros/fuerte
         parser.add_option("-j", "--parallel", dest="jobs", default=1,
                           help="How many parallel threads to use for installing",
                           action="store")
+        parser.add_option("--shallow", dest="shallow", default=False,
+                          help="Checkout only latest revision if possible",
+                          action="store_true")
         (options, args) = parser.parse_args(argv)
         if len(args) < 1:
             target_path = '.'
@@ -472,6 +475,7 @@ $ %(prog)s init ~/fuerte /opt/ros/fuerte
         install_success = multiproject_cmd.cmd_install_or_update(
             config,
             robust=False,
+            shallow=options.shallow,
             num_threads=int(options.jobs))
 
         if not install_success:
@@ -659,6 +663,18 @@ $ %(progname)s foreach --git 'git status'
         parser.add_option("--bzr", dest="bzr", default=False,
                           help="run command in bzr entries",
                           action="store_true")
+        parser.add_option("-m", "--timeout", dest="timeout",
+                          default=None,
+                          help="How long to wait for each repo before failing [seconds]",
+                          action="store", type=float)
+        parser.add_option("-j", "--parallel", dest="jobs",
+                          default=1,
+                          help="How many parallel threads to use for running the custom commands",
+                          action="store")
+        parser.add_option("-v", "--verbose", dest="verbose",
+                          default=False,
+                          help="Whether to print out more information",
+                          action="store_true")
         # -t option required here for help but used one layer above
         # see cli_common
         parser.add_option("-t", "--target-workspace", dest="workspace",
@@ -705,14 +721,18 @@ $ %(progname)s foreach --git 'git status'
         outputs = multiproject_cmd.cmd_foreach(config,
                                                command=command,
                                                localnames=localnames,
+                                               num_threads=int(options.jobs),
+                                               timeout=options.timeout,
                                                scm_types=scm_types,
-                                               shell=options.shell)
+                                               shell=options.shell,
+                                               verbose=options.verbose)
 
         def add_localname_prefix(localname, lines):
             return ['[%s] %s' % (localname, line) for line in lines]
 
         for output in outputs:
             localname = output['entry'].get_local_name()
+            rc = output['returncode']
             if options.show_stdout:
                 if output['stdout'] is None:
                     continue
@@ -721,20 +741,24 @@ $ %(progname)s foreach --git 'git status'
                 sys.stdout.write('\n'.join(lines))
                 sys.stdout.write('\n')
             if options.show_stderr:
-                if output['stderr'] is None:
+                lines = []
+                if output['stderr'] is not None:
+                    lines += output['stderr'].strip().split('\n')
+                if rc != 0:
+                    lines +=  ['Command failed with return code [%s]' % rc]
+                if not lines:
                     continue
-                lines = output['stderr'].strip().split('\n')
                 lines = add_localname_prefix(localname, lines)
                 sys.stderr.write('\n'.join(lines))
                 sys.stderr.write('\n')
-        return 0
+        return 0 if all([o['returncode'] == 0  for o in outputs]) else 1
 
     def cmd_status(self, target_path, argv, config=None):
         parser = OptionParser(usage="usage: %s status [localname]* " % self.progname,
                               description=__MULTIPRO_CMD_DICT__["status"] +
                               ". The status columns meanings are as the respective SCM defines them.",
                               epilog="""See: http://www.ros.org/wiki/rosinstall for details""")
-        parser.add_option("--untracked", dest="untracked",
+        parser.add_option("-u", "--untracked", dest="untracked",
                           default=False,
                           help="Also shows untracked files",
                           action="store_true")
@@ -1132,6 +1156,10 @@ $ %(prog)s info --only=path,cur_uri,cur_revision robot_model geometry
             "--data-only", dest="data_only", default=False,
             help="Does not provide explanations",
             action="store_true")
+        parser.add_option(
+            "-s", "--short", dest="short", default=False,
+            help="Shows simplified version info table.",
+            action="store_true")
         parser.add_option(
             "--only", dest="only", default=False,
             help="Shows comma-separated lists of only given comma-separated attribute(s).",
@@ -1181,6 +1209,7 @@ $ %(prog)s info --only=path,cur_uri,cur_revision robot_model geometry
             if only_options == '':
                 parser.error('No valid options given')
             lines = get_info_table_raw_csv(config,
+                                           parser,
                                            properties=only_options,
                                            localnames=args)
             print('\n'.join(lines))
@@ -1203,12 +1232,17 @@ $ %(prog)s info --only=path,cur_uri,cur_revision robot_model geometry
                                 options.data_only))
             return 0
 
+        columns = None
+        if options.short:
+            columns = ['localname', 'status', 'version']
+
         header = 'workspace: %s' % (target_path)
         print(header)
         table = get_info_table(config.get_base_path(),
                                outputs,
                                options.data_only,
-                               reverse=reverse)
+                               reverse=reverse,
+                               selected_headers=columns)
         if table is not None and table != '':
            print("\n%s" % table)
 
@@ -1218,7 +1252,8 @@ $ %(prog)s info --only=path,cur_uri,cur_revision robot_model geometry
                                    outputs2,
                                    options.data_only,
                                    reverse=reverse,
-                                   unmanaged=True)
+                                   unmanaged=True,
+                                   selected_headers=columns)
             if table2 is not None and table2 != '':
                 print("\nAlso detected these repositories in the workspace, add using '%s scrape' or '%s set':\n\n%s" % (self.progname, self.progname, table2))
 
